"""! @file filters.py
@brief Filter library for the application
@author xlanro00
"""

import numpy as np
#import matplotlib.pyplot as plt
import cv2 as cv


# Parent class for all the filters
class filter:
    '''
    Parent class for all the filters
    '''
    def __init__(self, img):
        self.img = img


class convolve(filter):
    ''' Convolve using custom kernel, 
    if no kernel is given, use default 3x3 kernel for averaging'''  
    def __init__(self, img):
        super().__init__(img)

    def apply(self, params):
        # Set default values
        ksize = int(params["ksize"]) if params["ksize"] else 3
        kernel = np.array(params["kernel"]) if params["kernel"] else np.ones((ksize, ksize), np.float32) / np.sqrt(ksize)

        #print("with params: " + " ksize: " + str(ksize) + " kernel: \n" + str(kernel))
        self.img = cv.filter2D(self.img, -1, kernel)


class blur(filter):
    def __init__(self, img):
        super().__init__(img)

    def apply(self, params):
        # Set default values
        if(params["anchor"]):
            try:
                anchor = tuple(map(int, params["anchor"].split(',')))
            except AttributeError:
                anchor = tuple(params["anchor"])
        else:
            anchor = (-1, -1)
        ksize = int(params["ksize"]) if params["ksize"] else 3

        
        #print("with params: " + " ksize: " + str(ksize) + " anchor: " + str(anchor))
        self.img = cv.blur(self.img, ksize=(ksize, ksize), anchor=anchor)

class gaussian(filter):
    def __init__(self, img):
        super().__init__(img)

    def apply(self, params):
        # Set default values
        ksize = int(params["ksize"]) if params["ksize"] else 3
        sigmaX = float(params["sigmaX"]) if params["sigmaX"] else 0
        sigmaY = float(params["sigmaY"]) if params["sigmaY"] else 0

        #print("with params: " + " ksize: " + str(ksize) + " sigmaX: " + str(sigmaX) + " sigmaY: " + str(sigmaY))
        self.img = cv.GaussianBlur(self.img, (ksize, ksize), sigmaX, sigmaY)


class median(filter):
    def __init__(self, img):
        super().__init__(img)

    def apply(self, params):
        # Set default values
        ksize = int(params["ksize"]) if params["ksize"] else 3

        #print("with params: " + " ksize: " + str(ksize))
        self.img = cv.medianBlur(self.img, ksize)


class bilateral(filter):
    def __init__(self, img):
        super().__init__(img)

    def apply(self, params):
        # Set default values
        d = int(params["d"]) if params["d"] else 1
        sigmaColor = int(params["sigmaColor"]) if params["sigmaColor"] else 75
        sigmaSpace = int(params["sigmaSpace"]) if params["sigmaSpace"] else 75

        #print("with params: " + " d: " + str(d) + " sigmaColor: " + str(sigmaColor) + " sigmaSpace: " + str(sigmaSpace))
        self.img = cv.bilateralFilter(self.img, d, sigmaColor, sigmaSpace)


class denoise(filter):
    def __init__(self, img):
        super().__init__(img)

    def apply(self, params):
        # Set default values
        h = int(params["h"]) if params["h"] else 20
        tWS = int(params["templateWindowSize"]) if params["templateWindowSize"] else 7
        sWS = int(params["searchWindowSize"]) if params["searchWindowSize"] else 21

        #print("with params: " + " h: " + str(h) + " tWS: " + str(tWS) + " sWS: " + str(sWS))
        self.img = np.uint8(self.img)
        self.img = cv.fastNlMeansDenoising(self.img, h, tWS, sWS)


class sharpen(filter):
    def __init__(self, img):
        super().__init__(img)

    def apply(self, params):
        # Set default values
        kernel = np.matrix(params["kernel"]) if params["kernel"] else np.array(
            [[0, -1, 0], [-1, 5, -1], [0, -1, 0]])

        #print("with params: " + " kernel: \n" + str(kernel))
        self.img = cv.filter2D(self.img, ddepth=-1, kernel=kernel)


class morph(filter):
    ''' General morphological operations.

    Can be used with MORPH_OPEN, MORPH_CLOSE, MORPH_DILATE, MORPH_ERODE as op.
    '''
    def __init__(self, img):
        super().__init__(img)

    def apply(self, params):
        # Set default values
        kernel = np.matrix(params["kernel"]) if params["kernel"] else np.ones((3, 3), np.uint8)
        iterations = int(params["iterations"]) if params["iterations"] else 1
        op = getattr(cv, params["op"]) if params["op"] else cv.MORPH_OPEN
        if(params["anchor"]):
            try:
                anchor = tuple(map(int, params["anchor"].split(',')))
            except AttributeError:
                anchor = tuple(params["anchor"])
        else:
            anchor = (-1, -1)


        #print("with params: " + " kernel: \n" + str(kernel) + " anchor: " + str(anchor) + " iterations: " + str(iterations) + " op: " + str(op))
        self.img = cv.morphologyEx(
            self.img, op=op, kernel=kernel, anchor=anchor, iterations=iterations)
